@@ -5,6 +5,7 @@ from typing import TYPE_CHECKING
 import pytest
 
 from poetry.packages import Locker
+from poetry.toml import TOMLFile
 
 
 if TYPE_CHECKING:
@@ -67,8 +68,6 @@ All set!
 def test_check_invalid(
     mocker: MockerFixture, tester: CommandTester, fixture_dir: FixtureDirGetter
 ) -> None:
-    from poetry.toml import TOMLFile
-
     mocker.patch(
         "poetry.poetry.Poetry.file",
         return_value=TOMLFile(fixture_dir("invalid_pyproject") / "pyproject.toml"),
@@ -77,11 +76,15 @@ def test_check_invalid(
 
     tester.execute("--lock")
 
-    expected = """\
-Error: 'description' is a required property
+    fastjsonschema_error = "data must contain ['description'] properties"
+    custom_error = "The fields ['description'] are required in package mode."
+    expected_template = """\
+Error: {schema_error}
 Error: Project name (invalid) is same as one of its dependencies
 Error: Unrecognized classifiers: ['Intended Audience :: Clowns'].
 Error: Declared README file does not exist: never/exists.md
+Error: Invalid source "not-exists" referenced in dependencies.
+Error: Invalid source "not-exists2" referenced in dependencies.
 Error: poetry.lock was not found.
 Warning: A wildcard Python dependency is ambiguous.\
  Consider specifying a more explicit one.
@@ -93,16 +96,39 @@ Warning: Deprecated classifier\
  'Topic :: Communications :: Chat :: AOL Instant Messenger'.\
  Must be removed.
 """
+    expected = {
+        expected_template.format(schema_error=schema_error)
+        for schema_error in (fastjsonschema_error, custom_error)
+    }
 
-    assert tester.io.fetch_error() == expected
+    assert tester.io.fetch_error() in expected
 
 
 def test_check_private(
     mocker: MockerFixture, tester: CommandTester, fixture_dir: FixtureDirGetter
 ) -> None:
     mocker.patch(
-        "poetry.factory.Factory.locate",
-        return_value=fixture_dir("private_pyproject") / "pyproject.toml",
+        "poetry.poetry.Poetry.file",
+        return_value=TOMLFile(fixture_dir("private_pyproject") / "pyproject.toml"),
+        new_callable=mocker.PropertyMock,
+    )
+
+    tester.execute()
+
+    expected = """\
+All set!
+"""
+
+    assert tester.io.fetch_output() == expected
+
+
+def test_check_non_package_mode(
+    mocker: MockerFixture, tester: CommandTester, fixture_dir: FixtureDirGetter
+) -> None:
+    mocker.patch(
+        "poetry.poetry.Poetry.file",
+        return_value=TOMLFile(fixture_dir("non_package_mode") / "pyproject.toml"),
+        new_callable=mocker.PropertyMock,
     )
 
     tester.execute()
@@ -129,8 +155,6 @@ def test_check_lock_missing(
     expected: str,
     expected_status: int,
 ) -> None:
-    from poetry.toml import TOMLFile
-
     mocker.patch(
         "poetry.poetry.Poetry.file",
         return_value=TOMLFile(fixture_dir("private_pyproject") / "pyproject.toml"),
@@ -165,8 +189,8 @@ def test_check_lock_outdated(
     tester = command_tester_factory("check", poetry=poetry_with_outdated_lockfile)
     status_code = tester.execute(options)
     expected = (
-        "Error: poetry.lock is not consistent with pyproject.toml. "
-        "Run `poetry lock [--no-update]` to fix it.\n"
+        "Error: pyproject.toml changed significantly since poetry.lock was last generated. "
+        "Run `poetry lock [--no-update]` to fix the lock file.\n"
     )
 
     assert tester.io.fetch_error() == expected
