@@ -2,20 +2,22 @@ from __future__ import annotations
 
 import os
 import shutil
+import subprocess
 import sys
 
 from pathlib import Path
 from typing import TYPE_CHECKING
+from typing import Any
 
 import pytest
 
 from cleo.testers.command_tester import CommandTester
 from packaging.utils import canonicalize_name
+from poetry.core.utils.helpers import module_name
 
+from poetry.console.application import Application
 from poetry.console.commands.init import InitCommand
 from poetry.repositories import RepositoryPool
-from poetry.utils._compat import decode
-from tests.helpers import PoetryTestApplication
 from tests.helpers import get_package
 
 
@@ -26,18 +28,19 @@ if TYPE_CHECKING:
     from poetry.core.packages.package import Package
     from pytest_mock import MockerFixture
 
+    from poetry.config.config import Config
     from poetry.poetry import Poetry
+    from tests.helpers import PoetryTestApplication
     from tests.helpers import TestRepository
     from tests.types import FixtureDirGetter
 
 
 @pytest.fixture
 def source_dir(tmp_path: Path) -> Iterator[Path]:
-    cwd = os.getcwd()
-
+    cwd = Path.cwd()
     try:
-        os.chdir(str(tmp_path))
-        yield Path(tmp_path.as_posix())
+        os.chdir(tmp_path)
+        yield tmp_path
     finally:
         os.chdir(cwd)
 
@@ -53,8 +56,7 @@ def patches(mocker: MockerFixture, source_dir: Path, repo: TestRepository) -> No
 
 @pytest.fixture
 def tester(patches: None) -> CommandTester:
-    # we need a test application without poetry here.
-    app = PoetryTestApplication(None)
+    app = Application()
     return CommandTester(app.find("init"))
 
 
@@ -85,7 +87,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -94,7 +95,7 @@ python = "~2.7 || ^3.6"
 
 def test_basic_interactive(
     tester: CommandTester, init_basic_inputs: str, init_basic_toml: str
-):
+) -> None:
     tester.execute(inputs=init_basic_inputs)
     assert init_basic_toml in tester.io.fetch_output()
 
@@ -105,8 +106,9 @@ def test_noninteractive(
     poetry: Poetry,
     repo: TestRepository,
     tmp_path: Path,
-):
+) -> None:
     command = app.find("init")
+    assert isinstance(command, InitCommand)
     command._pool = poetry.pool
 
     repo.add_package(get_package("pytest", "3.6.0"))
@@ -127,7 +129,9 @@ def test_noninteractive(
     assert 'pytest = "^3.6.0"' in toml_content
 
 
-def test_interactive_with_dependencies(tester: CommandTester, repo: TestRepository):
+def test_interactive_with_dependencies(
+    tester: CommandTester, repo: TestRepository
+) -> None:
     repo.add_package(get_package("django-pendulum", "0.1.6-pre4"))
     repo.add_package(get_package("pendulum", "2.0.0"))
     repo.add_package(get_package("pytest", "3.6.0"))
@@ -165,7 +169,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -182,7 +185,7 @@ pytest = "^3.6.0"
 # Regression test for https://github.com/python-poetry/poetry/issues/2355
 def test_interactive_with_dependencies_and_no_selection(
     tester: CommandTester, repo: TestRepository
-):
+) -> None:
     repo.add_package(get_package("django-pendulum", "0.1.6-pre4"))
     repo.add_package(get_package("pendulum", "2.0.0"))
     repo.add_package(get_package("pytest", "3.6.0"))
@@ -214,7 +217,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -223,7 +225,7 @@ python = "~2.7 || ^3.6"
     assert expected in tester.io.fetch_output()
 
 
-def test_empty_license(tester: CommandTester):
+def test_empty_license(tester: CommandTester) -> None:
     inputs = [
         "my-package",  # Package name
         "1.2.3",  # Version
@@ -245,7 +247,6 @@ version = "1.2.3"
 description = ""
 authors = ["Your Name <you@example.com>"]
 readme = "README.md"
-packages = [{{include = "my_package"}}]
 
 [tool.poetry.dependencies]
 python = "^{python}"
@@ -253,7 +254,9 @@ python = "^{python}"
     assert expected in tester.io.fetch_output()
 
 
-def test_interactive_with_git_dependencies(tester: CommandTester, repo: TestRepository):
+def test_interactive_with_git_dependencies(
+    tester: CommandTester, repo: TestRepository
+) -> None:
     repo.add_package(get_package("pendulum", "2.0.0"))
     repo.add_package(get_package("pytest", "3.6.0"))
 
@@ -284,7 +287,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -323,7 +325,8 @@ _generate_choice_list_packages_params: list[list[Package]] = [
 
 @pytest.fixture(params=_generate_choice_list_packages_params)
 def _generate_choice_list_packages(request: FixtureRequest) -> list[Package]:
-    return request.param
+    packages: list[Package] = request.param
+    return packages
 
 
 @pytest.mark.parametrize("package_name", ["flask", "Flask", "flAsK"])
@@ -331,8 +334,9 @@ def test_generate_choice_list(
     tester: CommandTester,
     package_name: str,
     _generate_choice_list_packages: list[Package],
-):
+) -> None:
     init_command = tester.command
+    assert isinstance(init_command, InitCommand)
 
     packages = _generate_choice_list_packages
     choices = init_command._generate_choice_list(
@@ -344,7 +348,7 @@ def test_generate_choice_list(
 
 def test_interactive_with_git_dependencies_with_reference(
     tester: CommandTester, repo: TestRepository
-):
+) -> None:
     repo.add_package(get_package("pendulum", "2.0.0"))
     repo.add_package(get_package("pytest", "3.6.0"))
 
@@ -375,7 +379,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -390,7 +393,7 @@ pytest = "^3.6.0"
 
 def test_interactive_with_git_dependencies_and_other_name(
     tester: CommandTester, repo: TestRepository
-):
+) -> None:
     repo.add_package(get_package("pendulum", "2.0.0"))
     repo.add_package(get_package("pytest", "3.6.0"))
 
@@ -421,7 +424,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -439,7 +441,7 @@ def test_interactive_with_directory_dependency(
     repo: TestRepository,
     source_dir: Path,
     fixture_dir: FixtureDirGetter,
-):
+) -> None:
     repo.add_package(get_package("pendulum", "2.0.0"))
     repo.add_package(get_package("pytest", "3.6.0"))
 
@@ -473,7 +475,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -490,7 +491,7 @@ def test_interactive_with_directory_dependency_and_other_name(
     repo: TestRepository,
     source_dir: Path,
     fixture_dir: FixtureDirGetter,
-):
+) -> None:
     repo.add_package(get_package("pendulum", "2.0.0"))
     repo.add_package(get_package("pytest", "3.6.0"))
 
@@ -524,7 +525,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -542,7 +542,7 @@ def test_interactive_with_file_dependency(
     repo: TestRepository,
     source_dir: Path,
     fixture_dir: FixtureDirGetter,
-):
+) -> None:
     repo.add_package(get_package("pendulum", "2.0.0"))
     repo.add_package(get_package("pytest", "3.6.0"))
 
@@ -576,7 +576,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -591,10 +590,7 @@ pytest = "^3.6.0"
 
 def test_interactive_with_wrong_dependency_inputs(
     tester: CommandTester, repo: TestRepository
-):
-    repo.add_package(get_package("pendulum", "2.0.0"))
-    repo.add_package(get_package("pytest", "3.6.0"))
-
+) -> None:
     inputs = [
         "my-package",  # Package name
         "1.2.3",  # Version
@@ -603,10 +599,8 @@ def test_interactive_with_wrong_dependency_inputs(
         "MIT",  # License
         "^3.8",  # Python
         "",  # Interactive packages
+        "foo 1.19.2",
         "pendulum 2.0.0 foo",  # Package name and constraint (invalid)
-        "pendulum 2.0.0",  # Package name and constraint (invalid)
-        "pendulum 2.0.0",  # Package name and constraint (invalid)
-        "pendulum 2.0.0",  # Package name and constraint (invalid)
         "pendulum@^2.0.0",  # Package name and constraint (valid)
         "",  # End package selection
         "",  # Interactive dev packages
@@ -626,10 +620,10 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "^3.8"
+foo = "1.19.2"
 pendulum = "^2.0.0"
 
 [tool.poetry.group.dev.dependencies]
@@ -639,7 +633,7 @@ pytest = "3.6.0"
     assert expected in tester.io.fetch_output()
 
 
-def test_python_option(tester: CommandTester):
+def test_python_option(tester: CommandTester) -> None:
     inputs = [
         "my-package",  # Package name
         "1.2.3",  # Version
@@ -660,7 +654,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -669,7 +662,7 @@ python = "~2.7 || ^3.6"
     assert expected in tester.io.fetch_output()
 
 
-def test_predefined_dependency(tester: CommandTester, repo: TestRepository):
+def test_predefined_dependency(tester: CommandTester, repo: TestRepository) -> None:
     repo.add_package(get_package("pendulum", "2.0.0"))
 
     inputs = [
@@ -693,7 +686,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -705,7 +697,7 @@ pendulum = "^2.0.0"
 
 def test_predefined_and_interactive_dependencies(
     tester: CommandTester, repo: TestRepository
-):
+) -> None:
     repo.add_package(get_package("pendulum", "2.0.0"))
     repo.add_package(get_package("pyramid", "1.10"))
 
@@ -735,7 +727,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -746,7 +737,7 @@ python = "~2.7 || ^3.6"
     assert 'pyramid = "^1.10"' in output
 
 
-def test_predefined_dev_dependency(tester: CommandTester, repo: TestRepository):
+def test_predefined_dev_dependency(tester: CommandTester, repo: TestRepository) -> None:
     repo.add_package(get_package("pytest", "3.6.0"))
 
     inputs = [
@@ -771,7 +762,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -785,7 +775,7 @@ pytest = "^3.6.0"
 
 def test_predefined_and_interactive_dev_dependencies(
     tester: CommandTester, repo: TestRepository
-):
+) -> None:
     repo.add_package(get_package("pytest", "3.6.0"))
     repo.add_package(get_package("pytest-requests", "0.2.0"))
 
@@ -815,7 +805,6 @@ description = "This is a description"
 authors = ["Your Name <you@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "~2.7 || ^3.6"
@@ -831,7 +820,7 @@ pytest-requests = "^0.2.0"
     assert 'pytest = "^3.6.0"' in output
 
 
-def test_predefined_all_options(tester: CommandTester, repo: TestRepository):
+def test_predefined_all_options(tester: CommandTester, repo: TestRepository) -> None:
     repo.add_package(get_package("pendulum", "2.0.0"))
     repo.add_package(get_package("pytest", "3.6.0"))
 
@@ -862,7 +851,6 @@ description = "This is a description"
 authors = ["Foo Bar <foo@example.com>"]
 license = "MIT"
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "^3.8"
@@ -876,8 +864,10 @@ pytest = "^3.6.0"
     assert expected in output
 
 
-def test_add_package_with_extras_and_whitespace(tester: CommandTester):
-    result = tester.command._parse_requirements(["databases[postgresql, sqlite]"])
+def test_add_package_with_extras_and_whitespace(tester: CommandTester) -> None:
+    command = tester.command
+    assert isinstance(command, InitCommand)
+    result = command._parse_requirements(["databases[postgresql, sqlite]"])
 
     assert result[0]["name"] == "databases"
     assert len(result[0]["extras"]) == 2
@@ -890,13 +880,13 @@ def test_init_existing_pyproject_simple(
     source_dir: Path,
     init_basic_inputs: str,
     init_basic_toml: str,
-):
+) -> None:
     pyproject_file = source_dir / "pyproject.toml"
     existing_section = """
 [tool.black]
 line-length = 88
 """
-    pyproject_file.write_text(decode(existing_section))
+    pyproject_file.write_text(existing_section)
     tester.execute(inputs=init_basic_inputs)
     assert f"{existing_section}\n{init_basic_toml}" in pyproject_file.read_text()
 
@@ -908,7 +898,7 @@ def test_init_existing_pyproject_consistent_linesep(
     init_basic_inputs: str,
     init_basic_toml: str,
     linesep: str,
-):
+) -> None:
     pyproject_file = source_dir / "pyproject.toml"
     existing_section = """
 [tool.black]
@@ -930,13 +920,13 @@ def test_init_non_interactive_existing_pyproject_add_dependency(
     source_dir: Path,
     init_basic_inputs: str,
     repo: TestRepository,
-):
+) -> None:
     pyproject_file = source_dir / "pyproject.toml"
     existing_section = """
 [tool.black]
 line-length = 88
 """
-    pyproject_file.write_text(decode(existing_section))
+    pyproject_file.write_text(existing_section)
 
     repo.add_package(get_package("foo", "1.19.2"))
 
@@ -955,7 +945,6 @@ version = "0.1.0"
 description = ""
 authors = ["Your Name <you@example.com>"]
 readme = "README.md"
-packages = [{include = "my_package"}]
 
 [tool.poetry.dependencies]
 python = "^3.6"
@@ -966,14 +955,14 @@ foo = "^1.19.2"
 
 def test_init_existing_pyproject_with_build_system_fails(
     tester: CommandTester, source_dir: Path, init_basic_inputs: str
-):
+) -> None:
     pyproject_file = source_dir / "pyproject.toml"
     existing_section = """
 [build-system]
 requires = ["setuptools >= 40.6.0", "wheel"]
 build-backend = "setuptools.build_meta"
 """
-    pyproject_file.write_text(decode(existing_section))
+    pyproject_file.write_text(existing_section)
     tester.execute(inputs=init_basic_inputs)
     assert (
         tester.io.fetch_error().strip()
@@ -996,18 +985,34 @@ build-backend = "setuptools.build_meta"
         "   foo 2.0  ",
     ],
 )
-def test__validate_package_valid(name: str | None):
+def test_validate_package_valid(name: str | None) -> None:
     assert InitCommand._validate_package(name) == name
 
 
 @pytest.mark.parametrize(
     "name", ["foo bar 2.0", "   foo bar 2.0   ", "foo bar foobar 2.0"]
 )
-def test__validate_package_invalid(name: str):
+def test_validate_package_invalid(name: str) -> None:
     with pytest.raises(ValueError):
         assert InitCommand._validate_package(name)
 
 
+@pytest.mark.parametrize(
+    "author",
+    [
+        str(b"Jos\x65\xcc\x81 Duarte", "utf-8"),
+        str(b"Jos\xc3\xa9 Duarte", "utf-8"),
+    ],
+)
+def test_validate_author(author: str) -> None:
+    """
+    This test was added following issue #8779, hence, we're looking to see if the test
+    no longer throws an exception, hence the seemingly "useless" test of just running
+    the method.
+    """
+    InitCommand._validate_author(author, "")
+
+
 @pytest.mark.parametrize(
     "package_name, include",
     (
@@ -1022,7 +1027,7 @@ def test_package_include(
     tester: CommandTester,
     package_name: str,
     include: str | None,
-):
+) -> None:
     tester.execute(
         inputs="\n".join(
             (
@@ -1039,9 +1044,8 @@ def test_package_include(
         ),
     )
 
-    if include is None:
-        packages = ""
-    else:
+    packages = ""
+    if include and module_name(package_name) != include:
         packages = f'packages = [{{include = "{include}"}}]\n'
 
     expected = (
@@ -1057,3 +1061,61 @@ def test_package_include(
         f'python = "^3.10"\n'
     )
     assert expected in tester.io.fetch_output()
+
+
+@pytest.mark.parametrize(
+    ["prefer_active", "python"],
+    [
+        (True, "1.1"),
+        (False, f"{sys.version_info[0]}.{sys.version_info[1]}"),
+    ],
+)
+def test_respect_prefer_active_on_init(
+    prefer_active: bool,
+    python: str,
+    config: Config,
+    mocker: MockerFixture,
+    tester: CommandTester,
+    source_dir: Path,
+) -> None:
+    from poetry.utils.env import GET_PYTHON_VERSION_ONELINER
+
+    orig_check_output = subprocess.check_output
+
+    def mock_check_output(cmd: str, *_: Any, **__: Any) -> str:
+        if GET_PYTHON_VERSION_ONELINER in cmd:
+            return "1.1.1"
+
+        result: str = orig_check_output(cmd, *_, **__)
+        return result
+
+    mocker.patch("subprocess.check_output", side_effect=mock_check_output)
+
+    config.config["virtualenvs"]["prefer-active-python"] = prefer_active
+    pyproject_file = source_dir / "pyproject.toml"
+
+    tester.execute(
+        "--author 'Your Name <you@example.com>' --name 'my-package'",
+        interactive=False,
+    )
+
+    expected = f"""\
+[tool.poetry.dependencies]
+python = "^{python}"
+"""
+
+    assert expected in pyproject_file.read_text()
+
+
+def test_get_pool(mocker: MockerFixture, source_dir: Path) -> None:
+    """
+    Since we are mocking _get_pool() in the other tests, we at least should make
+    sure it works in general. See https://github.com/python-poetry/poetry/issues/8634.
+    """
+    mocker.patch("pathlib.Path.cwd", return_value=source_dir)
+
+    app = Application()
+    command = app.find("init")
+    assert isinstance(command, InitCommand)
+    pool = command._get_pool()
+    assert pool.repositories
