@@ -23,8 +23,8 @@ def test_dependency_does_not_match_root_python_constraint(
     add_to_repo(repo, "foo", "1.0.0", python="<3.5")
 
     error = """\
-The current project's Python requirement (>=3.6,<4.0) is not compatible with some of\
- the required packages Python requirement:
+The current project's supported Python range (>=3.6,<4.0) is not compatible with some\
+ of the required packages Python requirement:
   - foo requires Python <3.5, so it will not be satisfied for Python >=3.6,<4.0
 
 Because no versions of foo match !=1.0.0
