@@ -1,12 +1,13 @@
 from __future__ import annotations
 
-from pathlib import Path
 from subprocess import CalledProcessError
 from typing import TYPE_CHECKING
+from typing import Any
 
 import pytest
 
 from cleo.io.null_io import NullIO
+from packaging.utils import canonicalize_name
 from poetry.core.packages.dependency import Dependency
 from poetry.core.packages.directory_dependency import DirectoryDependency
 from poetry.core.packages.file_dependency import FileDependency
@@ -18,8 +19,11 @@ from poetry.core.packages.vcs_dependency import VCSDependency
 from poetry.factory import Factory
 from poetry.inspection.info import PackageInfo
 from poetry.packages import DependencyPackage
+from poetry.puzzle.provider import IncompatibleConstraintsError
 from poetry.puzzle.provider import Provider
+from poetry.repositories.exceptions import PackageNotFound
 from poetry.repositories.repository import Repository
+from poetry.repositories.repository_pool import Priority
 from poetry.repositories.repository_pool import RepositoryPool
 from poetry.utils.env import EnvCommandError
 from poetry.utils.env import MockEnv as BaseMockEnv
@@ -29,12 +33,14 @@ from tests.helpers import get_dependency
 if TYPE_CHECKING:
     from pytest_mock import MockerFixture
 
+    from tests.types import FixtureDirGetter
+
 
 SOME_URL = "https://example.com/path.tar.gz"
 
 
 class MockEnv(BaseMockEnv):
-    def run(self, bin: str, *args: str) -> None:
+    def run(self, bin: str, *args: str, **kwargs: Any) -> str:
         raise EnvCommandError(CalledProcessError(1, "python", output=""))
 
 
@@ -67,15 +73,7 @@ def provider(root: ProjectPackage, pool: RepositoryPool) -> Provider:
         (Dependency("foo", "<2"), [Package("foo", "1")]),
         (Dependency("foo", "<2", extras=["bar"]), [Package("foo", "1")]),
         (Dependency("foo", ">=1"), [Package("foo", "2"), Package("foo", "1")]),
-        (
-            Dependency("foo", ">=1a"),
-            [
-                Package("foo", "3a"),
-                Package("foo", "2"),
-                Package("foo", "2a"),
-                Package("foo", "1"),
-            ],
-        ),
+        (Dependency("foo", ">=1a"), [Package("foo", "2"), Package("foo", "1")]),
         (
             Dependency("foo", ">=1", allows_prereleases=True),
             [
@@ -101,6 +99,7 @@ def test_search_for(
     repository.add_package(foo2a)
     repository.add_package(foo2)
     repository.add_package(foo3a)
+
     assert provider.search_for(dependency) == expected
 
 
@@ -117,7 +116,7 @@ def test_search_for(
             Dependency("foo", ">=2"),
             URLDependency("foo", SOME_URL),
             [Package("foo", "3")],
-            [],
+            [Package("foo", "3")],
         ),
         (
             Dependency("foo", ">=1", extras=["bar"]),
@@ -164,7 +163,7 @@ def test_search_for_direct_origin_and_extras(
 
 
 @pytest.mark.parametrize("value", [True, False])
-def test_search_for_vcs_retains_develop_flag(provider: Provider, value: bool):
+def test_search_for_vcs_retains_develop_flag(provider: Provider, value: bool) -> None:
     dependency = VCSDependency(
         "demo", "git", "https://github.com/demo/demo.git", develop=value
     )
@@ -172,7 +171,7 @@ def test_search_for_vcs_retains_develop_flag(provider: Provider, value: bool):
     assert package.develop == value
 
 
-def test_search_for_vcs_setup_egg_info(provider: Provider):
+def test_search_for_vcs_setup_egg_info(provider: Provider) -> None:
     dependency = VCSDependency("demo", "git", "https://github.com/demo/demo.git")
 
     package = provider.search_for_direct_origin_dependency(dependency)
@@ -190,7 +189,7 @@ def test_search_for_vcs_setup_egg_info(provider: Provider):
     }
 
 
-def test_search_for_vcs_setup_egg_info_with_extras(provider: Provider):
+def test_search_for_vcs_setup_egg_info_with_extras(provider: Provider) -> None:
     dependency = VCSDependency(
         "demo", "git", "https://github.com/demo/demo.git", extras=["foo"]
     )
@@ -210,7 +209,7 @@ def test_search_for_vcs_setup_egg_info_with_extras(provider: Provider):
     }
 
 
-def test_search_for_vcs_read_setup(provider: Provider, mocker: MockerFixture):
+def test_search_for_vcs_read_setup(provider: Provider, mocker: MockerFixture) -> None:
     mocker.patch("poetry.utils.env.EnvManager.get", return_value=MockEnv())
 
     dependency = VCSDependency("demo", "git", "https://github.com/demo/demo.git")
@@ -232,7 +231,7 @@ def test_search_for_vcs_read_setup(provider: Provider, mocker: MockerFixture):
 
 def test_search_for_vcs_read_setup_with_extras(
     provider: Provider, mocker: MockerFixture
-):
+) -> None:
     mocker.patch("poetry.utils.env.EnvManager.get", return_value=MockEnv())
 
     dependency = VCSDependency(
@@ -252,7 +251,7 @@ def test_search_for_vcs_read_setup_with_extras(
 
 def test_search_for_vcs_read_setup_raises_error_if_no_version(
     provider: Provider, mocker: MockerFixture
-):
+) -> None:
     mocker.patch(
         "poetry.inspection.info.get_pep517_metadata",
         return_value=PackageInfo(name="demo", version=None),
@@ -265,15 +264,12 @@ def test_search_for_vcs_read_setup_raises_error_if_no_version(
 
 
 @pytest.mark.parametrize("directory", ["demo", "non-canonical-name"])
-def test_search_for_directory_setup_egg_info(provider: Provider, directory: str):
+def test_search_for_directory_setup_egg_info(
+    provider: Provider, directory: str, fixture_dir: FixtureDirGetter
+) -> None:
     dependency = DirectoryDependency(
         "demo",
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "git"
-        / "github.com"
-        / "demo"
-        / directory,
+        fixture_dir("git") / "github.com" / "demo" / directory,
     )
 
     package = provider.search_for_direct_origin_dependency(dependency)
@@ -291,15 +287,12 @@ def test_search_for_directory_setup_egg_info(provider: Provider, directory: str)
     }
 
 
-def test_search_for_directory_setup_egg_info_with_extras(provider: Provider):
+def test_search_for_directory_setup_egg_info_with_extras(
+    provider: Provider, fixture_dir: FixtureDirGetter
+) -> None:
     dependency = DirectoryDependency(
         "demo",
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "git"
-        / "github.com"
-        / "demo"
-        / "demo",
+        fixture_dir("git") / "github.com" / "demo" / "demo",
         extras=["foo"],
     )
 
@@ -319,21 +312,13 @@ def test_search_for_directory_setup_egg_info_with_extras(provider: Provider):
 
 
 @pytest.mark.parametrize("directory", ["demo", "non-canonical-name"])
-def test_search_for_directory_setup_with_base(provider: Provider, directory: str):
+def test_search_for_directory_setup_with_base(
+    provider: Provider, directory: str, fixture_dir: FixtureDirGetter
+) -> None:
     dependency = DirectoryDependency(
         "demo",
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "git"
-        / "github.com"
-        / "demo"
-        / directory,
-        base=Path(__file__).parent.parent
-        / "fixtures"
-        / "git"
-        / "github.com"
-        / "demo"
-        / directory,
+        fixture_dir("git") / "github.com" / "demo" / directory,
+        base=fixture_dir("git") / "github.com" / "demo" / directory,
     )
 
     package = provider.search_for_direct_origin_dependency(dependency)
@@ -349,29 +334,17 @@ def test_search_for_directory_setup_with_base(provider: Provider, directory: str
         "foo": [get_dependency("cleo")],
         "bar": [get_dependency("tomlkit")],
     }
-    assert package.root_dir == (
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "git"
-        / "github.com"
-        / "demo"
-        / directory
-    )
+    assert package.root_dir == (fixture_dir("git") / "github.com" / "demo" / directory)
 
 
 def test_search_for_directory_setup_read_setup(
-    provider: Provider, mocker: MockerFixture
-):
+    provider: Provider, mocker: MockerFixture, fixture_dir: FixtureDirGetter
+) -> None:
     mocker.patch("poetry.utils.env.EnvManager.get", return_value=MockEnv())
 
     dependency = DirectoryDependency(
         "demo",
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "git"
-        / "github.com"
-        / "demo"
-        / "demo",
+        fixture_dir("git") / "github.com" / "demo" / "demo",
     )
 
     package = provider.search_for_direct_origin_dependency(dependency)
@@ -390,18 +363,13 @@ def test_search_for_directory_setup_read_setup(
 
 
 def test_search_for_directory_setup_read_setup_with_extras(
-    provider: Provider, mocker: MockerFixture
-):
+    provider: Provider, mocker: MockerFixture, fixture_dir: FixtureDirGetter
+) -> None:
     mocker.patch("poetry.utils.env.EnvManager.get", return_value=MockEnv())
 
     dependency = DirectoryDependency(
         "demo",
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "git"
-        / "github.com"
-        / "demo"
-        / "demo",
+        fixture_dir("git") / "github.com" / "demo" / "demo",
         extras=["foo"],
     )
 
@@ -420,15 +388,12 @@ def test_search_for_directory_setup_read_setup_with_extras(
     }
 
 
-def test_search_for_directory_setup_read_setup_with_no_dependencies(provider: Provider):
+def test_search_for_directory_setup_read_setup_with_no_dependencies(
+    provider: Provider, fixture_dir: FixtureDirGetter
+) -> None:
     dependency = DirectoryDependency(
         "demo",
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "git"
-        / "github.com"
-        / "demo"
-        / "no-dependencies",
+        fixture_dir("git") / "github.com" / "demo" / "no-dependencies",
     )
 
     package = provider.search_for_direct_origin_dependency(dependency)
@@ -439,10 +404,12 @@ def test_search_for_directory_setup_read_setup_with_no_dependencies(provider: Pr
     assert package.extras == {}
 
 
-def test_search_for_directory_poetry(provider: Provider):
+def test_search_for_directory_poetry(
+    provider: Provider, fixture_dir: FixtureDirGetter
+) -> None:
     dependency = DirectoryDependency(
         "project-with-extras",
-        Path(__file__).parent.parent / "fixtures" / "project_with_extras",
+        fixture_dir("project_with_extras"),
     )
 
     package = provider.search_for_direct_origin_dependency(dependency)
@@ -461,16 +428,19 @@ def test_search_for_directory_poetry(provider: Provider):
         get_dependency("cachy", ">=0.2.0"),
         get_dependency("pendulum", ">=1.4.4"),
     ]
-    assert package.extras == {
-        "extras-a": [get_dependency("pendulum", ">=1.4.4")],
-        "extras-b": [get_dependency("cachy", ">=0.2.0")],
-    }
+    extras_a = canonicalize_name("extras-a")
+    extras_b = canonicalize_name("extras-b")
+    assert set(package.extras) == {extras_a, extras_b}
+    assert set(package.extras[extras_a]) == {get_dependency("pendulum", ">=1.4.4")}
+    assert set(package.extras[extras_b]) == {get_dependency("cachy", ">=0.2.0")}
 
 
-def test_search_for_directory_poetry_with_extras(provider: Provider):
+def test_search_for_directory_poetry_with_extras(
+    provider: Provider, fixture_dir: FixtureDirGetter
+) -> None:
     dependency = DirectoryDependency(
         "project-with-extras",
-        Path(__file__).parent.parent / "fixtures" / "project_with_extras",
+        fixture_dir("project_with_extras"),
         extras=["extras_a"],
     )
 
@@ -490,19 +460,19 @@ def test_search_for_directory_poetry_with_extras(provider: Provider):
         get_dependency("cachy", ">=0.2.0"),
         get_dependency("pendulum", ">=1.4.4"),
     ]
-    assert package.extras == {
-        "extras-a": [get_dependency("pendulum", ">=1.4.4")],
-        "extras-b": [get_dependency("cachy", ">=0.2.0")],
-    }
+    extras_a = canonicalize_name("extras-a")
+    extras_b = canonicalize_name("extras-b")
+    assert set(package.extras) == {extras_a, extras_b}
+    assert set(package.extras[extras_a]) == {get_dependency("pendulum", ">=1.4.4")}
+    assert set(package.extras[extras_b]) == {get_dependency("cachy", ">=0.2.0")}
 
 
-def test_search_for_file_sdist(provider: Provider):
+def test_search_for_file_sdist(
+    provider: Provider, fixture_dir: FixtureDirGetter
+) -> None:
     dependency = FileDependency(
         "demo",
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "distributions"
-        / "demo-0.1.0.tar.gz",
+        fixture_dir("distributions") / "demo-0.1.0.tar.gz",
     )
 
     package = provider.search_for_direct_origin_dependency(dependency)
@@ -527,13 +497,12 @@ def test_search_for_file_sdist(provider: Provider):
     }
 
 
-def test_search_for_file_sdist_with_extras(provider: Provider):
+def test_search_for_file_sdist_with_extras(
+    provider: Provider, fixture_dir: FixtureDirGetter
+) -> None:
     dependency = FileDependency(
         "demo",
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "distributions"
-        / "demo-0.1.0.tar.gz",
+        fixture_dir("distributions") / "demo-0.1.0.tar.gz",
         extras=["foo"],
     )
 
@@ -559,13 +528,12 @@ def test_search_for_file_sdist_with_extras(provider: Provider):
     }
 
 
-def test_search_for_file_wheel(provider: Provider):
+def test_search_for_file_wheel(
+    provider: Provider, fixture_dir: FixtureDirGetter
+) -> None:
     dependency = FileDependency(
         "demo",
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "distributions"
-        / "demo-0.1.0-py2.py3-none-any.whl",
+        fixture_dir("distributions") / "demo-0.1.0-py2.py3-none-any.whl",
     )
 
     package = provider.search_for_direct_origin_dependency(dependency)
@@ -590,13 +558,12 @@ def test_search_for_file_wheel(provider: Provider):
     }
 
 
-def test_search_for_file_wheel_with_extras(provider: Provider):
+def test_search_for_file_wheel_with_extras(
+    provider: Provider, fixture_dir: FixtureDirGetter
+) -> None:
     dependency = FileDependency(
         "demo",
-        Path(__file__).parent.parent
-        / "fixtures"
-        / "distributions"
-        / "demo-0.1.0-py2.py3-none-any.whl",
+        fixture_dir("distributions") / "demo-0.1.0-py2.py3-none-any.whl",
         extras=["foo"],
     )
 
@@ -622,6 +589,36 @@ def test_search_for_file_wheel_with_extras(provider: Provider):
     }
 
 
+def test_complete_package_merges_same_source_and_no_source(
+    provider: Provider, root: ProjectPackage
+) -> None:
+    foo_no_source_1 = get_dependency("foo", ">=1")
+    foo_source_1 = get_dependency("foo", "!=1.1.*")
+    foo_source_1.source_name = "source"
+    foo_source_2 = get_dependency("foo", "!=1.2.*")
+    foo_source_2.source_name = "source"
+    foo_no_source_2 = get_dependency("foo", "<2")
+
+    root.add_dependency(foo_no_source_1)
+    root.add_dependency(foo_source_1)
+    root.add_dependency(foo_source_2)
+    root.add_dependency(foo_no_source_2)
+
+    complete_package = provider.complete_package(
+        DependencyPackage(root.to_dependency(), root)
+    )
+
+    requires = complete_package.package.all_requires
+    assert len(requires) == 1
+    assert requires[0].source_name == "source"
+    assert str(requires[0].constraint) in {
+        ">=1,<1.1 || >=1.3,<2",
+        ">=1,<1.1.dev0 || >=1.3.dev0,<2",
+        ">=1,<1.1.0 || >=1.3.0,<2",
+        ">=1,<1.1.0.dev0 || >=1.3.0.dev0,<2",
+    }
+
+
 def test_complete_package_does_not_merge_different_source_names(
     provider: Provider, root: ProjectPackage
 ) -> None:
@@ -633,40 +630,80 @@ def test_complete_package_does_not_merge_different_source_names(
     root.add_dependency(foo_source_1)
     root.add_dependency(foo_source_2)
 
+    with pytest.raises(IncompatibleConstraintsError) as e:
+        provider.complete_package(DependencyPackage(root.to_dependency(), root))
+
+    expected = """\
+Incompatible constraints in requirements of root (1.2.3):
+foo ; source=source_2
+foo ; source=source_1"""
+
+    assert str(e.value) == expected
+
+
+def test_complete_package_merges_same_source_type_and_no_source(
+    provider: Provider, root: ProjectPackage, fixture_dir: FixtureDirGetter
+) -> None:
+    project_dir = fixture_dir("with_conditional_path_deps")
+    path = (project_dir / "demo_one").as_posix()
+
+    root.add_dependency(Factory.create_dependency("demo", ">=1.0"))
+    root.add_dependency(Factory.create_dependency("demo", {"path": path}))
+    root.add_dependency(Factory.create_dependency("demo", {"path": path}))  # duplicate
+    root.add_dependency(Factory.create_dependency("demo", "<2.0"))
+
     complete_package = provider.complete_package(
         DependencyPackage(root.to_dependency(), root)
     )
 
     requires = complete_package.package.all_requires
-    assert len(requires) == 2
-    assert {requires[0].source_name, requires[1].source_name} == {
-        "source_1",
-        "source_2",
-    }
+    assert len(requires) == 1
+    assert requires[0].source_url == path
+    assert str(requires[0].constraint) == "1.2.3"
 
 
-def test_complete_package_preserves_source_type(
-    provider: Provider, root: ProjectPackage
+def test_complete_package_does_not_merge_different_source_types(
+    provider: Provider, root: ProjectPackage, fixture_dir: FixtureDirGetter
 ) -> None:
-    fixtures = Path(__file__).parent.parent / "fixtures"
-    project_dir = fixtures.joinpath("with_conditional_path_deps")
+    project_dir = fixture_dir("with_conditional_path_deps")
     for folder in ["demo_one", "demo_two"]:
         path = (project_dir / folder).as_posix()
         root.add_dependency(Factory.create_dependency("demo", {"path": path}))
 
-    complete_package = provider.complete_package(
-        DependencyPackage(root.to_dependency(), root)
-    )
+    with pytest.raises(IncompatibleConstraintsError) as e:
+        provider.complete_package(DependencyPackage(root.to_dependency(), root))
 
-    requires = complete_package.package.all_requires
-    assert len(requires) == 2
-    assert {requires[0].source_url, requires[1].source_url} == {
-        project_dir.joinpath("demo_one").as_posix(),
-        project_dir.joinpath("demo_two").as_posix(),
-    }
+    expected = f"""\
+Incompatible constraints in requirements of root (1.2.3):
+demo @ {project_dir.as_uri()}/demo_two (1.2.3)
+demo @ {project_dir.as_uri()}/demo_one (1.2.3)"""
+
+    assert str(e.value) == expected
 
 
-def test_complete_package_preserves_source_type_with_subdirectories(
+def test_complete_package_does_not_merge_different_source_type_and_name(
+    provider: Provider, root: ProjectPackage, fixture_dir: FixtureDirGetter
+) -> None:
+    project_dir = fixture_dir("with_conditional_path_deps")
+    path = (project_dir / "demo_one").as_posix()
+
+    dep_with_source_name = Factory.create_dependency("demo", ">=1.0")
+    dep_with_source_name.source_name = "source"
+    root.add_dependency(dep_with_source_name)
+    root.add_dependency(Factory.create_dependency("demo", {"path": path}))
+
+    with pytest.raises(IncompatibleConstraintsError) as e:
+        provider.complete_package(DependencyPackage(root.to_dependency(), root))
+
+    expected = f"""\
+Incompatible constraints in requirements of root (1.2.3):
+demo @ {project_dir.as_uri()}/demo_one (1.2.3)
+demo (>=1.0) ; source=source"""
+
+    assert str(e.value) == expected
+
+
+def test_complete_package_does_not_merge_different_subdirectories(
     provider: Provider, root: ProjectPackage
 ) -> None:
     dependency_one = Factory.create_dependency(
@@ -683,34 +720,19 @@ def test_complete_package_preserves_source_type_with_subdirectories(
             "subdirectory": "one-copy",
         },
     )
-    dependency_two = Factory.create_dependency(
-        "two",
-        {"git": "https://github.com/demo/subdirectories.git", "subdirectory": "two"},
-    )
 
-    root.add_dependency(
-        Factory.create_dependency(
-            "one",
-            {
-                "git": "https://github.com/demo/subdirectories.git",
-                "subdirectory": "one",
-            },
-        )
-    )
+    root.add_dependency(dependency_one)
     root.add_dependency(dependency_one_copy)
-    root.add_dependency(dependency_two)
 
-    complete_package = provider.complete_package(
-        DependencyPackage(root.to_dependency(), root)
-    )
+    with pytest.raises(IncompatibleConstraintsError) as e:
+        provider.complete_package(DependencyPackage(root.to_dependency(), root))
 
-    requires = complete_package.package.all_requires
-    assert len(requires) == 3
-    assert {r.to_pep_508() for r in requires} == {
-        dependency_one.to_pep_508(),
-        dependency_one_copy.to_pep_508(),
-        dependency_two.to_pep_508(),
-    }
+    expected = """\
+Incompatible constraints in requirements of root (1.2.3):
+one @ git+https://github.com/demo/subdirectories.git#subdirectory=one-copy (1.0.0)
+one @ git+https://github.com/demo/subdirectories.git#subdirectory=one (1.0.0)"""
+
+    assert str(e.value) == expected
 
 
 @pytest.mark.parametrize("source_name", [None, "repo"])
@@ -721,7 +743,7 @@ def test_complete_package_with_extras_preserves_source_name(
     package_b = Package("B", "1.0")
     dep = get_dependency("B", "^1.0", optional=True)
     package_a.add_dependency(dep)
-    package_a.extras = {"foo": [dep]}
+    package_a.extras = {canonicalize_name("foo"): [dep]}
     repository.add_package(package_a)
     repository.add_package(package_b)
 
@@ -744,13 +766,13 @@ def test_complete_package_with_extras_preserves_source_name(
 @pytest.mark.parametrize("with_extra", [False, True])
 def test_complete_package_fetches_optional_vcs_dependency_only_if_requested(
     provider: Provider, repository: Repository, mocker: MockerFixture, with_extra: bool
-):
+) -> None:
     optional_vcs_dependency = Factory.create_dependency(
         "demo", {"git": "https://github.com/demo/demo.git", "optional": True}
     )
     package = Package("A", "1.0", features=["foo"] if with_extra else [])
     package.add_dependency(optional_vcs_dependency)
-    package.extras["foo"] = [optional_vcs_dependency]
+    package.extras = {canonicalize_name("foo"): [optional_vcs_dependency]}
     repository.add_package(package)
 
     spy = mocker.spy(provider, "_search_for_vcs")
@@ -761,3 +783,99 @@ def test_complete_package_fetches_optional_vcs_dependency_only_if_requested(
         spy.assert_called()
     else:
         spy.assert_not_called()
+
+
+def test_complete_package_finds_locked_package_in_explicit_source(
+    root: ProjectPackage, pool: RepositoryPool
+) -> None:
+    package = Package("a", "1.0", source_reference="explicit")
+    explicit_repo = Repository("explicit")
+    explicit_repo.add_package(package)
+    pool.add_repository(explicit_repo, priority=Priority.EXPLICIT)
+
+    root_dependency = get_dependency("a", ">0")
+    root_dependency.source_name = "explicit"
+    root.add_dependency(root_dependency)
+    locked_package = Package("a", "1.0", source_reference="explicit")
+    provider = Provider(root, pool, NullIO(), locked=[locked_package])
+    provider.complete_package(DependencyPackage(root.to_dependency(), root))
+
+    # transitive dependency without explicit source
+    dependency = get_dependency("a", ">=1")
+
+    locked = provider.get_locked(dependency)
+    assert locked is not None
+    provider.complete_package(locked)  # must not fail
+
+
+def test_complete_package_finds_locked_package_in_other_source(
+    root: ProjectPackage, repository: Repository, pool: RepositoryPool
+) -> None:
+    package = Package("a", "1.0")
+    repository.add_package(package)
+    explicit_repo = Repository("explicit")
+    pool.add_repository(explicit_repo)
+
+    root_dependency = get_dependency("a", ">0")  # no explicit source
+    root.add_dependency(root_dependency)
+    locked_package = Package("a", "1.0", source_reference="explicit")  # explicit source
+    provider = Provider(root, pool, NullIO(), locked=[locked_package])
+    provider.complete_package(DependencyPackage(root.to_dependency(), root))
+
+    # transitive dependency without explicit source
+    dependency = get_dependency("a", ">=1")
+
+    locked = provider.get_locked(dependency)
+    assert locked is not None
+    provider.complete_package(locked)  # must not fail
+
+
+def test_complete_package_raises_packagenotfound_if_locked_source_not_available(
+    root: ProjectPackage, pool: RepositoryPool, provider: Provider
+) -> None:
+    locked_package = Package("a", "1.0", source_reference="outdated")
+    provider = Provider(root, pool, NullIO(), locked=[locked_package])
+    provider.complete_package(DependencyPackage(root.to_dependency(), root))
+
+    # transitive dependency without explicit source
+    dependency = get_dependency("a", ">=1")
+
+    locked = provider.get_locked(dependency)
+    assert locked is not None
+    with pytest.raises(PackageNotFound):
+        provider.complete_package(locked)
+
+
+def test_source_dependency_is_satisfied_by_direct_origin(
+    provider: Provider, repository: Repository
+) -> None:
+    direct_origin_package = Package("foo", "1.1", source_type="url")
+    repository.add_package(Package("foo", "1.0"))
+    provider._direct_origin_packages = {"foo": direct_origin_package}
+    dep = Dependency("foo", ">=1")
+
+    assert provider.search_for(dep) == [direct_origin_package]
+
+
+def test_explicit_source_dependency_is_not_satisfied_by_direct_origin(
+    provider: Provider, repository: Repository
+) -> None:
+    repo_package = Package("foo", "1.0")
+    repository.add_package(repo_package)
+    provider._direct_origin_packages = {"foo": Package("foo", "1.1", source_type="url")}
+    dep = Dependency("foo", ">=1")
+    dep.source_name = repository.name
+
+    assert provider.search_for(dep) == [repo_package]
+
+
+def test_source_dependency_is_not_satisfied_by_incompatible_direct_origin(
+    provider: Provider, repository: Repository
+) -> None:
+    repo_package = Package("foo", "2.0")
+    repository.add_package(repo_package)
+    provider._direct_origin_packages = {"foo": Package("foo", "1.0", source_type="url")}
+    dep = Dependency("foo", ">=2")
+    dep.source_name = repository.name
+
+    assert provider.search_for(dep) == [repo_package]
